/*Light Reaction Game
By: Stuart O'Connor


*/
#include <LiquidCrystal.h>

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(7, 8, 9, 10, 11, 12);

// Define Pins
int L1 = 2;
int L2 = 3;
int L3 = 4;
int L4 = 5;
int L5 = 6;
int player1Button = 15;
int player2Button = 14;
int buzzer = 13;

//Initialize Variables
bool print = false;
bool canClear = true;

//flow variables
int players = 0;
bool started = false;
bool buttonPressLegal = true;
long StartTime = 0;

//Input variables
bool leftButtonIsPressed = false;
bool leftButtonWasPressedLastFrame = false;
bool rightButtonIsPressed = false;
bool rightButtonWasPressedLastFrame = false;

void setup() {
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);

  //Pin setup
  pinMode(L1, OUTPUT);
  pinMode(L2, OUTPUT);
  pinMode(L3, OUTPUT);
  pinMode(L4, OUTPUT);
  pinMode(L5, OUTPUT);
  pinMode(player1Button, INPUT_PULLUP);
  pinMode(player2Button, INPUT_PULLUP);
  pinMode(buzzer, OUTPUT);
}

void loop(){
  
  ButtonInput();
  
  if(players == 0){
    int players = playerSelect();
  }
  else{
    if(!started){
      howToPlay();
    }
    else if (players == 1){
      if(LeftButtonDown()){
        StopTimer(0);
      }
    }
    else if(players == 2){
      if(LeftButtonDown()){
        StopTimer(0);
      }
      else if(RightButtonDown()){
        StopTimer(1);
      }
    }
  }

}

void lightSequence(){
  started = true;
  digitalWrite(L1, HIGH);
  playBuzzer();
  delay(500);
  digitalWrite(L2, HIGH);
  playBuzzer();
  delay(500);
  digitalWrite(L3,HIGH);
  playBuzzer();
  delay(500);
  digitalWrite(L4,HIGH);
  playBuzzer();
  delay(500);
  digitalWrite(L5, HIGH);
  playBuzzer();
  delay(random(1000, 5000));
  digitalWrite(L1, LOW);
  digitalWrite(L2, LOW);
  digitalWrite(L3, LOW);
  digitalWrite(L4, LOW);
  digitalWrite(L5, LOW);
  StartTime = millis();
  ButtonInput();
}

//Stops the reaction time timer.
//buttonIndex is the number of the button that was pressed. This determines which player has won.
void StopTimer(int buttonIndex) {
  if (buttonPressLegal)
  {
    float reactionTime = (millis() - StartTime);
    reactionTime = reactionTime/1000;
    switch (buttonIndex)
    {
      case 0:
        //left won. Point to left on LCD Screen
        lcd.clear();
        lcd.print(reactionTime + String("s"));
        lcd.setCursor(0,1);
        lcd.print("<-----");
        break;
      case 1:
        //right won. Point right on LCD Screen
        lcd.clear();
        lcd.print(reactionTime + String("s"));
        lcd.setCursor(0,1);
        lcd.print("          ----->");
        break;
    }
  }

  delay(5000);
  started = false;
  players = 0;
}

int playerSelect() {
  if(canClear == true){
    lcd.clear();
    canClear = false;
  }
  lcd.print("1 or 2 Players?");
  lcd.setCursor(0,1);
  lcd.print("<-- 1      2 -->");
  if(LeftButtonDown()){
    players = 1;
  }
  else if(RightButtonDown()){
    players = 2;
  }
  return players;
}

void playBuzzer(){
  digitalWrite(buzzer, HIGH);
  delay(500);
  digitalWrite(buzzer, LOW);
}

void howToPlay(){
  canClear = true;
  lcd.clear();
  lcd.write("Hit the button");
  lcd.setCursor(0, 1);
  lcd.write("when lights off");
  delay(5000);
  lightSequence();
}

void ButtonInput(){
  leftButtonWasPressedLastFrame = leftButtonIsPressed;
  rightButtonWasPressedLastFrame = rightButtonIsPressed;

  leftButtonIsPressed = !digitalRead(player1Button);
  rightButtonIsPressed = !digitalRead(player2Button);
}

//Returns true during the tick the user pressed the left button.
bool LeftButtonDown(){
  return leftButtonIsPressed && !leftButtonWasPressedLastFrame;
}

//Returns true during the tick the user pressed the right button.
bool RightButtonDown(){
  return rightButtonIsPressed && !rightButtonWasPressedLastFrame;
}